#include <stdio.h>
#include <stdlib.h>
#include <string.h>

void shell_sort(char ** C, int size, int recl)		//Big-O of shell is "n-log(n)"
{
	int div = size / 2;
	char temp[recl];
	int i = 0;
	int j = 0;
	while (div > 0)
	{
		for (i = div; i < size; i++)
		{
			strcpy(temp, C[i]);
			for(j = i; (j >= div) && (strcmp(C[j - div], temp) > 0); j = j - div)
				strcpy(C[j], C[j - div]);
			strcpy(C[j], temp);
		}
		div = div / 2;
	}
}

void bubble_sort(char ** C, int size, int recl)		//Big-O of bubble is "n-squared"
{
	_Bool done = 0;
	int i = 0;
	int j = 0;
	char temp[recl];

	for (i = 0; (i < size) && !done; i++)
	{
		done = 1;
		for (j = 0; j < size - i - 1; j++)
		{
			if (strcmp(C[j], C[j + 1]) > 0)
			{
				done = 0;
				strcpy(temp, C[j]);
				strcpy(C[j], C[j + 1]);
				strcpy(C[j + 1], temp);
			}
		}
	}
}

void insert_sort(char ** C, int size, int recl)		//Big-O of insert is "n-squared"
{
	int i = 0;
	int j = 0;
	char temp[recl];

	for(i = 1; i < size; i++)
	{
		strcpy(temp, C[i]);
		j = i;
		while((j > 0) && (strcmp(C[j - 1], temp) > 0))
		{
			strcpy(C[j], C[j - 1]);
			j--;
		}
		strcpy(C[j], temp);
	}
}

void display(char ** C, int size, int num_display)
{
	int i = 0; 
	if (num_display > size)
		num_display = size;
	for (i = 0; i < num_display; i++)
		printf("%d - %s\n", i, C[i]);
	printf("\n");
}

void read_data(FILE * ptr, char ** C, int size, int recl)
{
	int i = 0;
	if (recl == 21)
       		ptr = fopen("chars1000000.dat", "r");
	else
       		ptr = fopen("longchars1000000.dat", "r");

        if (ptr == NULL)			//make sure file got opened okay
                printf("File forked up. Call Karl. Now!\n");

	for (i = 0; i < size; i++)
		fscanf(ptr, "%s", C[i]);
}


int main(int argc, char ** argv)
{
/*	First argument is number of records to read in from master large file, should be <= 1MM.
	Second argument is number of records to display, should be <= first arg.
	Third argument is 21 or 81, for size of random string.
	Fourth argument is type of sort. 1=shell, 2=bubble, 3=insert
*/
	int size = atoi(argv[1]);		//number of records to read in from giant file
	int num_display = atoi(argv[2]);	//num_display should be < size
	int rec_len = atoi(argv[3]);		//rec_len should be 21 or 81, depending on which file to read
	int sort_type = atoi(argv[4]);		//sort_type, 1=shell, 2=bubble, 3=insert
        
	int i = 0;
	FILE * fp;

	//create 2-D character array for string data
	char ** Names;	
	Names = malloc(rec_len * size);		//allocate memory for array of char-pointers (c-strings)		
	for (i = 0; i < size; i++)		//allocate memory for individual c-strings
    		Names[i] = malloc(rec_len);

	read_data(fp, Names, size, rec_len);
	display(Names, size, num_display);

	printf("\nPress Enter to continue: ");	//utilitarian junk
	getchar();
	printf("\n");

	switch (sort_type)
	{
		case 1:	shell_sort(Names, size, rec_len);
			break;
		case 2:	bubble_sort(Names, size, rec_len);
			break;
		case 3:	insert_sort(Names, size, rec_len);
			break;
		default:
			sort_type = 1;
	}
	display(Names, size, num_display);

return 0;
}
